
class bombaatCloudError(Exception):
    """Base exception for cloud."""
    pass

class bombaatSSOError(bombaatCloudError):
    """Exception for SSO."""
    pass

class SAMLParserError(bombaatSSOError):
    """SAML related error"""
    pass

class SSOConnectionError(bombaatSSOError):
    """Raised when the provided SSO Auth failed or could not be validated."""
    pass

class SSOLoginError(bombaatSSOError):
    pass

class SSOFileReadError(bombaatSSOError):
    """Exception raised when there is config file issue."""
    pass

class FileNotFoundError(bombaatSSOError):
    """Exception raised when there is config file issue."""
    pass

class ProfileNotFound(bombaatSSOError):
    """Exception raised when profile not found in config"""
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Main Module"""

import sys
import logging

from bombaat import __version__
from bombaat.errors import bombaatCloudError
from bombaat.argparser import create_argument_parser

logger = logging.getLogger(__name__)

def print_version() -> None:
    sys.stdout.write(f'\nbombaat version {__version__}' + '\n')


def main() -> None:
    arg_parser = create_argument_parser()
    cmdline_arguments = arg_parser.parse_args()

    try:
        if hasattr(cmdline_arguments, 'func'):
            cmdline_arguments.func(cmdline_arguments)
        elif hasattr(cmdline_arguments, 'version'):
            print_version()
        else:
            # user has not provided a subcommand, let's print the help
            arg_parser.print_help()
            sys.exit(1)
    except bombaatCloudError as err:
        logger.info('\nFailed to run bombaat CLI due to an exception: %s'% (
                                                                    str(err),))
        sys.exit(1)


if __name__=='__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Metadata for bombaat."""

import os
import sys
import logging

from appdirs import AppDirs
from colorlog import ColoredFormatter

__author__ = """Ravi Boodher"""
__email__ = 'boodher@gmail.com'
__version__ = '1.0.1'
__status__ = "Production"
__bombaat_home__ = os.environ.get(
    'BOMBAAT_HOME', AppDirs('bombaat').user_data_dir)  # type: str
__default_download_host__ = 'https://bombaat.s3.ap-south-1.amazonaws.com'
DEBUG = False

# Create a root logger
logger = logging.getLogger('bombaat')
log_level = logging.INFO
log_format = '%(log_color)s%(message)s%(reset)s'
formatter = ColoredFormatter(log_format)
stream_handler = logging.StreamHandler()
stream_handler.setLevel(log_level)
stream_handler.setFormatter(formatter)
logger.setLevel(log_level)
logger.addHandler(stream_handler)

def exception_handler(exception_type, exception, traceback):
    # All your trace are belong to us!
    # your format
    print(exception_type.__name__, exception)

sys.excepthook = exception_handler



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""SP SAML Login Handler."""

import os
import sys
import boto3
import getpass
import logging

import inquirer
from blessed import Terminal
from botocore.configloader import load_config
from botocore.exceptions import ConfigNotFound
from botocore.session import Session, get_session
from awscli.customizations.configure.writer import ConfigFileWriter

from bombaat.utils import ask
from bombaat.saml import SAMLParser
from bombaat.utils import read_yaml
from bombaat.utils import MorningSun
from bombaat.errors import ProfileNotFound
from bombaat.utils import get_cloud_profiles
from bombaat.utils import datetime_utc_to_local
from bombaat.configure import write_credentials
from bombaat.configure import configure_default_profile

logger = logging.getLogger(__name__)

class Login(object):
    """Class to initiate AWS authentication with SAML token."""

    def __init__(self):
        self._session = get_session()
        self.cloud_profiles = get_cloud_profiles()
        self._config_writer = ConfigFileWriter()

    def main(self) -> None:
        selected_profile = self._get_login_profile()
        profile_details = self.cloud_profiles['login_urls'][selected_profile]
        _session = Session(profile=selected_profile)
        config = _session.get_scoped_config()
        login_url = profile_details['login_url']
        if config.get('username') is None:
            msg = (f'Username not found for the profile {selected_profile}, '
                'you can run `bombaat configure` to reconfigure the profile.')
            logger.error(msg)
            sys.exit(1)
        username = config['username']
        password = getpass.getpass(f'Enter password for the user {username}: ')
        get_saml = SAMLParser(login_url, username, password).run()
        aws_roles = get_saml['roles_arn']
        role_chosen = self._choose_role(aws_roles)
        cred = self._assume_role(role_chosen, get_saml['principal_arn'][0],
                                                    get_saml['saml_response'])
        new_values = cred['Credentials']
        write_credentials(_session, new_values, selected_profile)
        configure_default_profile()
        time = new_values['Expiration']
        time = datetime_utc_to_local(time)

        cloud_profile_friendly_name = profile_details['name']
        logger.info((f'Login successful! Your access for '
                f'{cloud_profile_friendly_name} expires at {time}!'))

    def _get_login_profile(self) -> str:
        try:
            config_file = self._session.get_config_variable('config_file')
            _config = load_config(config_file)
        except ConfigNotFound:
            _config = {'profiles': {}}

        available_profiles = list(_config['profiles'].keys())
        account_names = list(self.cloud_profiles['login_urls'].keys())
        login_profiles = list(set(available_profiles).intersection(
                                                                account_names))
        if len(login_profiles) == 0:
            logger.warning('\nNo cloud profile found, you can run'
                    ' `bombaat configure` command to setup a new profile.')
            sys.exit(1)
        msg = 'Select the cloud profile you would like to login'
        _answers = ask(msg, login_profiles)
        if _answers:
            _choice = _answers.get('choice').strip()
            return _choice

    @staticmethod
    def _choose_role(aws_roles: str) -> str:
        """Prompt user to choose role.

        :param role_arn: role amazon resource name
        :return: role arn selected by an user
        """
        if len(aws_roles) == 0:
            logger.warning('You do not have access to this cloud account.')
            sys.exit(1)

        if len(aws_roles) == 1:
            return aws_roles[0]

        short_names = [role.split('/')[1] for role in aws_roles]
        msg = 'Choose the role you would like to connect'
        _answers = ask(msg, short_names)
        if _answers:
            _choice = _answers.get('choice').strip().lower()
            for role in aws_roles:
                if _choice in role.lower(): return role
        else:
            # KeyboardInterrupts
            sys.exit(1)

    @staticmethod
    def _assume_role(role_arn: str,
                       principal_arn: str,
                       saml_response: str) -> dict:
        """Get temporary security credentials.

        Returns a set of temporary security credentials for users who have been
        authenticated via a SAML authentication response.

        :param role_arn: role amazon resource name
        :param principal_arn: principal name
        :param saml_response: SAML object to assume role with
        :param duration: session duration (default: 3600)
        :return: AWS session token
        """
        conn = boto3.client('sts')
        aws_session_token = conn.assume_role_with_saml(
            RoleArn=role_arn,
            PrincipalArn=principal_arn,
            SAMLAssertion=saml_response,
            DurationSeconds=28800,
        )
        return aws_session_token



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Argument Parser Module"""

import sys
import logging
import argparse

from typing import List, Text

from bombaat.login import Login
from bombaat.utils import print_msg
from bombaat.configure import Configure
from bombaat.driver import update_driver
from bombaat.profiles import download_profile
from bombaat.init import update_downloader_path

logger = logging.getLogger(__name__)

SubParsersAction = argparse._SubParsersAction

HELP_BLURB = (
    '\nTo see help text, you can run:\n'
    '\n'
    '  bombaat --help\n'
    '  bombaat <command> --help' + '\n'
)

def create_argument_parser() -> argparse.ArgumentParser:
    """Parse all the command line arguments."""

    parser = argparse.ArgumentParser(
        prog='bombaat',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description='bombaat command line interface. bombaat CLI enables '
                    'you to access AWS Cloud Accounts using Azure single '
                    'sign on service via `awscli` or `boto`.',)

    parser.add_argument(
        '--version',
        action='store_true',
        default=argparse.SUPPRESS,
        help='print installed bombaat CLI version',)

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parsers = [parent_parser]
    subparsers = parser.add_subparsers(help='bombaat CLI commands')

    init_subparser(subparsers, parents=parent_parsers)
    update_subparser(subparsers, parents=parent_parsers)
    configure_subparser(subparsers, parents=parent_parsers)
    login_subparser(subparsers, parents=parent_parsers)
    return parser

def init_subparser(subparsers: SubParsersAction,
                    parents: List[argparse.ArgumentParser]) -> None:
    """Init parsers."""

    login_parser = subparsers.add_parser(
        'init',
        parents=parents,
        help='Initialize downloader host.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,)
    login_parser.set_defaults(func=run_init)


def run_init(args: argparse.Namespace) -> None:
    update_downloader_path()


def update_subparser(subparsers: SubParsersAction,
                        parents: List[argparse.ArgumentParser]) -> None:
    """update parsers."""

    configure_parser = subparsers.add_parser(
        'update',
        conflict_handler='resolve',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        parents=parents,
        help='Install a new profile and drivers.',)

    configure_parser.add_argument('configure_option',
                                help= 'update bombaat config.',
                                choices=['all', 'profile', 'config',])

    configure_parser.set_defaults(func=run_update)


def run_update(args: argparse.Namespace) -> None:
    msg = f'Please wait while setup installs necessary configuration files.'
    print_msg(msg)
    sys.stdout.write('\n')
    if args.configure_option == 'all':
        update_driver()
        sys.stdout.write('\n')
        download_profile()
        print_msg('Update completed.\n')
        print_msg(HELP_BLURB)
    elif args.configure_option == 'profile':
        download_profile()
        print_msg('Profiles update completed.\n')
    elif args.configure_option == 'config':
        update_driver()
        print_msg('Config update completed.\n')


def configure_subparser(subparsers: SubParsersAction,
                    parents: List[argparse.ArgumentParser]) -> None:
    """Login parsers."""

    login_parser = subparsers.add_parser(
        'configure',
        parents=parents,
        help='Configure cloud profile.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,)
    login_parser.set_defaults(func=run_configure)


def run_configure(args: argparse.Namespace) -> None:
    Configure().main()


def login_subparser(subparsers: SubParsersAction,
                    parents: List[argparse.ArgumentParser]) -> None:
    """Login parsers."""

    login_parser = subparsers.add_parser(
        'login',
        parents=parents,
        help='Log in to a cloud profile.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,)
    login_parser.set_defaults(func=run_login)


def run_login(args: argparse.Namespace) -> None:
    Login().main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

install_requires = [
    'boto==2.49.0',
    'tqdm==4.50.2',
    'PyYAML==5.3.1',
    'boto3==1.16.43',
    'appdirs==1.4.4',
    'urllib3==1.25.8',
    'inquirer==2.7.0',
    'colorlog==4.6.2',
    'blessed==1.17.6',
    'awscli==1.18.205',
    'selenium==3.141.0',
    'botocore==1.19.45',
]

setup_options = dict(
    name="bombaat",
    version='1.0.6',
    author="Ravi Boodher",
    author_email="boodher@gmail.com",
    description="Bombaat CLI - awscli / boto AWS API access using Azure Active Directory (AD) service.",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=['tests*']),
    package_data={'': ['*']},
    include_package_data=True,
    install_requires=install_requires,
    extras_require={},
    entry_points={
            "console_scripts": [
                "bombaat=bombaat.__main__:main",
            ]
        },

    classifiers=[
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        'Environment :: Console',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
    ],
    python_requires='>=3.6',
)

setup(**setup_options)
